/* USBHID.CS
 *
 * Definitions for the USBHID interface DLL version 1.6.
 * Copyright (c) 2019-2024, CompuPhase, The Netherlands
 */

using System;
using System.Text;
using System.Runtime.InteropServices;

namespace USBHID {
  public enum LEDFunction : ushort { OFF = 0, ON, BLINK };
  public enum KbdLayout : int {
    BELGIAN = 2,
    CANADIAN = 4,
    DANISH = 6,
    FRENCH = 8,
    GERMAN = 9,
    INTERNATIONAL = 13,
    ITALIAN = 14,
    LATIN_AMERICAN = 17,
    NORWEGIAN = 19,
    PORTUGUESE = 22,
    SPANISH = 25,
    SWEDISH = 26,
    UK = 32,
    US = 33
  };
  public class usbhid {

    /* 32-bit definitions */
    [DllImport("usbhid32.dll", EntryPoint = "usbhid_Initialize", CallingConvention = CallingConvention.StdCall)]
    private static extern uint Initialize_32();

    [DllImport("usbhid32.dll", EntryPoint = "usbhid_Close", CallingConvention = CallingConvention.StdCall)]
    private static extern void Close_32();

    [DllImport("usbhid32.dll", EntryPoint = "usbhid_Version", CallingConvention = CallingConvention.StdCall)]
    private static extern uint Version_32();

    [DllImport("usbhid32.dll", EntryPoint = "usbhid_GetInfo", CallingConvention = CallingConvention.StdCall)]
    [return: MarshalAs(UnmanagedType.Bool)]
    private static extern bool GetInfo_32(uint DeviceIndex, ref uint SerialNumber, ref int Model);

    [DllImport("usbhid32.dll", EntryPoint = "usbhid_GetKeyString", CallingConvention = CallingConvention.StdCall)]
    [return: MarshalAs(UnmanagedType.Bool)]
    private static extern bool GetKeyString_32(uint DeviceIndex, uint SwitchIndex, StringBuilder KeyString, uint MaxLength);

    [DllImport("usbhid32.dll", EntryPoint = "usbhid_GetSwitchSerial", CallingConvention = CallingConvention.StdCall)]
    [return: MarshalAs(UnmanagedType.Bool)]
    private static extern bool GetSwitchSerial_32(uint DeviceIndex, uint SwitchIndex, ref uint SerialNumber);

    [DllImport("usbhid32.dll", EntryPoint = "usbhid_IsSwitchDown", CallingConvention = CallingConvention.StdCall)]
    [return: MarshalAs(UnmanagedType.Bool)]
    private static extern bool IsSwitchDown_32(uint DeviceIndex, uint SwitchIndex);

    [DllImport("usbhid32.dll", EntryPoint = "usbhid_SetDisabled", CallingConvention = CallingConvention.StdCall)]
    [return: MarshalAs(UnmanagedType.Bool)]
    private static extern bool SetDisabled_32(uint DeviceIndex, ushort Disabled);

    [DllImport("usbhid32.dll", EntryPoint = "usbhid_SetLED", CallingConvention = CallingConvention.StdCall)]
    [return: MarshalAs(UnmanagedType.Bool)]
    private static extern bool SetLED_32(uint DeviceIndex, LEDFunction Function, ushort Dimming);

    [DllImport("usbhid32.dll", EntryPoint = "usbhid_SetOutput", CallingConvention = CallingConvention.StdCall)]
    [return: MarshalAs(UnmanagedType.Bool)]
    private static extern bool SetOutput_32(uint DeviceIndex, uint OutputIndex, uint Value);

    [DllImport("usbhid32.dll", EntryPoint = "usbhid_SetLayout", CallingConvention = CallingConvention.StdCall)]
    [return: MarshalAs(UnmanagedType.Bool)]
    private static extern bool SetLayout_32(int Layout);

    /* 64-bit definitions */
    [DllImport("usbhid64.dll", EntryPoint = "usbhid_Initialize", CallingConvention = CallingConvention.StdCall)]
    private static extern uint Initialize_64();

    [DllImport("usbhid64.dll", EntryPoint = "usbhid_Close", CallingConvention = CallingConvention.StdCall)]
    private static extern void Close_64();

    [DllImport("usbhid64.dll", EntryPoint = "usbhid_Version", CallingConvention = CallingConvention.StdCall)]
    private static extern uint Version_64();

    [DllImport("usbhid64.dll", EntryPoint = "usbhid_GetInfo", CallingConvention = CallingConvention.StdCall)]
    [return: MarshalAs(UnmanagedType.Bool)]
    private static extern bool GetInfo_64(uint DeviceIndex, ref uint SerialNumber, ref int Model);

    [DllImport("usbhid64.dll", EntryPoint = "usbhid_GetKeyString", CallingConvention = CallingConvention.StdCall)]
    [return: MarshalAs(UnmanagedType.Bool)]
    private static extern bool GetKeyString_64(uint DeviceIndex, uint SwitchIndex, StringBuilder KeyString, uint MaxLength);

    [DllImport("usbhid64.dll", EntryPoint = "usbhid_GetSwitchSerial", CallingConvention = CallingConvention.StdCall)]
    [return: MarshalAs(UnmanagedType.Bool)]
    private static extern bool GetSwitchSerial_64(uint DeviceIndex, uint SwitchIndex, ref uint SerialNumber);

    [DllImport("usbhid64.dll", EntryPoint = "usbhid_IsSwitchDown", CallingConvention = CallingConvention.StdCall)]
    [return: MarshalAs(UnmanagedType.Bool)]
    private static extern bool IsSwitchDown_64(uint DeviceIndex, uint SwitchIndex);

    [DllImport("usbhid64.dll", EntryPoint = "usbhid_SetDisabled", CallingConvention = CallingConvention.StdCall)]
    [return: MarshalAs(UnmanagedType.Bool)]
    private static extern bool SetDisabled_64(uint DeviceIndex, ushort Disabled);

    [DllImport("usbhid64.dll", EntryPoint = "usbhid_SetLED", CallingConvention = CallingConvention.StdCall)]
    [return: MarshalAs(UnmanagedType.Bool)]
    private static extern bool SetLED_64(uint DeviceIndex, LEDFunction Function, ushort Dimming);

    [DllImport("usbhid64.dll", EntryPoint = "usbhid_SetOutput", CallingConvention = CallingConvention.StdCall)]
    [return: MarshalAs(UnmanagedType.Bool)]
    private static extern bool SetOutput_64(uint DeviceIndex, uint OutputIndex, uint Value);

    [DllImport("usbhid64.dll", EntryPoint = "usbhid_SetLayout", CallingConvention = CallingConvention.StdCall)]
    [return: MarshalAs(UnmanagedType.Bool)]
    private static extern bool SetLayout_64(int Layout);

    /* interface functions */
    private static bool is64Bit() {
      return (IntPtr.Size == 8);
    }

    public static uint Initialize() {
      return (is64Bit() ? Initialize_64() : Initialize_32());
    }

    public static void Close() {
      if (is64Bit()) Close_64(); else Close_32();
    }

    public static uint Version() {
      return (is64Bit() ? Version_64() : Version_32());
    }

    public static bool GetInfo(uint DeviceIndex, ref uint SerialNumber, ref int Model) {
      return (is64Bit() ? GetInfo_64(DeviceIndex, ref SerialNumber, ref Model) : GetInfo_32(DeviceIndex, ref SerialNumber, ref Model));
    }

    public static bool GetKeyString(uint DeviceIndex, uint SwitchIndex, StringBuilder KeyString, uint MaxLength) {
      return (is64Bit() ? GetKeyString_64(DeviceIndex, SwitchIndex, KeyString, MaxLength) : GetKeyString_32(DeviceIndex, SwitchIndex, KeyString, MaxLength));
    }

    public static bool GetSwitchSerial(uint DeviceIndex, uint SwitchIndex, ref uint SerialNumber) {
      return (is64Bit() ? GetSwitchSerial_64(DeviceIndex, SwitchIndex, ref SerialNumber) : GetSwitchSerial_32(DeviceIndex, SwitchIndex, ref SerialNumber));
    }

    public static bool IsSwitchDown(uint DeviceIndex, uint SwitchIndex) {
      return (is64Bit() ? IsSwitchDown_64(DeviceIndex, SwitchIndex) : IsSwitchDown_32(DeviceIndex, SwitchIndex));
    }

    public static bool SetDisabled(uint DeviceIndex, ushort Disabled) {
      return (is64Bit() ? SetDisabled_64(DeviceIndex, Disabled) : SetDisabled_32(DeviceIndex, Disabled));
    }

    public static bool SetLED(uint DeviceIndex, LEDFunction Function, ushort Dimming) {
      return (is64Bit() ? SetLED_64(DeviceIndex, Function, Dimming) : SetLED_32(DeviceIndex, Function, Dimming));
    }

    public static bool SetOutput(uint DeviceIndex, uint OutputIndex, uint Value) {
      return (is64Bit() ? SetOutput_64(DeviceIndex, OutputIndex, Value) : SetOutput_32(DeviceIndex, OutputIndex, Value));
    }

    public static bool SetLayout(int Layout) {
      return (is64Bit() ? SetLayout_64(Layout) : SetLayout_32(Layout));
    }
  }
}
