' USBHID.CS
'
' Definitions for the USBHID interface DLL version 1.6.
' Copyright (c) 2019-2024, CompuPhase, The Netherlands
'

Imports System
Imports System.Text
Imports System.Runtime.InteropServices

Public Class usbhid
    Public Enum LEDFunction As UShort
        Led_OFF = 0
        Led_ON = 1
        Led_BLINK = 2
    End Enum

    Public Enum KbdLayout As Integer
        LAYOUT_BELGIAN = 2
        LAYOUT_CANADIAN = 4
        LAYOUT_DANISH = 6
        LAYOUT_FRENCH = 8
        LAYOUT_GERMAN = 9
        LAYOUT_INTERNATIONAL = 13
        LAYOUT_ITALIAN = 14
        LAYOUT_LATIN_AMERICAN = 17
        LAYOUT_NORWEGIAN = 19
        LAYOUT_PORTUGUESE = 22
        LAYOUT_SPANISH = 25
        LAYOUT_SWEDISH = 26
        LAYOUT_UK = 32
        LAYOUT_US = 33
    End Enum

    ' 32-bit definitions
    <DllImport("usbhid32.dll", EntryPoint := "usbhid_Initialize", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Function Initialize_32() As UInteger
    End Function

    <DllImport("usbhid32.dll", EntryPoint := "usbhid_Close", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Sub Close_32()
    End Sub

    <DllImport("usbhid32.dll", EntryPoint := "usbhid_Version", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Function Version_32() As UInteger
    End Function

    <DllImport("usbhid32.dll", EntryPoint := "usbhid_GetInfo", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Function GetInfo_32(ByVal DeviceIndex As UInteger, ByRef SerialNumber As UInteger, ByRef Model As Integer) As Boolean
    End Function

    <DllImport("usbhid32.dll", EntryPoint := "usbhid_GetKeyString", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Function GetKeyString_32(ByVal DeviceIndex As UInteger, ByVal SwitchIndex As UInteger, ByVal KeyString As StringBuilder, ByVal MaxLength As UInteger) As Boolean
    End Function

    <DllImport("usbhid32.dll", EntryPoint := "usbhid_GetSwitchSerial", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Function GetSwitchSerial_32(ByVal DeviceIndex As UInteger, ByVal SwitchIndex As UInteger, ByRef SerialNumber As UInteger) As Boolean
    End Function

    <DllImport("usbhid32.dll", EntryPoint := "usbhid_IsSwitchDown", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Function IsSwitchDown_32(ByVal DeviceIndex As UInteger, ByVal SwitchIndex As UInteger) As Boolean
    End Function

    <DllImport("usbhid32.dll", EntryPoint := "usbhid_SetDisabled", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Function SetDisabled_32(ByVal DeviceIndex As UInteger, ByVal Disabled As UShort) As Boolean
    End Function

    <DllImport("usbhid32.dll", EntryPoint := "usbhid_SetLED", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Function SetLED_32(ByVal DeviceIndex As UInteger, ByVal LEDFunc As LEDFunction, ByVal Dimming As UShort) As Boolean
    End Function

    <DllImport("usbhid32.dll", EntryPoint := "usbhid_SetOutput", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Function SetOutput_32(ByVal DeviceIndex As UInteger, ByVal OutputIndex As UInteger, ByVal Value As UInteger) As Boolean
    End Function

    <DllImport("usbhid32.dll", EntryPoint := "usbhid_SetLayout", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Function SetLayout_32(ByVal Layout As Integer) As Boolean
    End Function

    ' 64-bit definitions
    <DllImport("usbhid64.dll", EntryPoint := "usbhid_Initialize", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Function Initialize_64() As UInteger
    End Function

    <DllImport("usbhid64.dll", EntryPoint := "usbhid_Close", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Sub Close_64()
    End Sub

    <DllImport("usbhid64.dll", EntryPoint := "usbhid_Version", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Function Version_64() As UInteger
    End Function

    <DllImport("usbhid64.dll", EntryPoint := "usbhid_GetInfo", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Function GetInfo_64(ByVal DeviceIndex As UInteger, ByRef SerialNumber As UInteger, ByRef Model As Integer) As Boolean
    End Function

    <DllImport("usbhid64.dll", EntryPoint := "usbhid_GetKeyString", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Function GetKeyString_64(ByVal DeviceIndex As UInteger, ByVal SwitchIndex As UInteger, ByVal KeyString As StringBuilder, ByVal MaxLength As UInteger) As Boolean
    End Function

    <DllImport("usbhid64.dll", EntryPoint := "usbhid_GetSwitchSerial", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Function GetSwitchSerial_64(ByVal DeviceIndex As UInteger, ByVal SwitchIndex As UInteger, ByRef SerialNumber As UInteger) As Boolean
    End Function

    <DllImport("usbhid64.dll", EntryPoint := "usbhid_IsSwitchDown", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Function IsSwitchDown_64(ByVal DeviceIndex As UInteger, ByVal SwitchIndex As UInteger) As Boolean;
    End Function

    <DllImport("usbhid64.dll", EntryPoint := "usbhid_SetDisabled", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Function SetDisabled_64(ByVal DeviceIndex As UInteger, ByVal Disabled As UShort) As Boolean
    End Function

    <DllImport("usbhid64.dll", EntryPoint := "usbhid_SetLED", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Function SetLED_64(ByVal DeviceIndex As UInteger, ByVal LEDFunc As LEDFunction, ByVal Dimming As UShort) As Boolean
    End Function

    <DllImport("usbhid64.dll", EntryPoint := "usbhid_SetOutput", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Function SetOutput_64(ByVal DeviceIndex As UInteger, ByVal OutputIndex As UInteger, ByVal Value As UInteger) As Boolean
    End Function

    <DllImport("usbhid64.dll", EntryPoint := "usbhid_SetLayout", CallingConvention := CallingConvention.StdCall)> _
    Private Shared Function SetLayout_64(ByVal Layout As Integer) As Boolean
    End Function

    ' interface functions
    Private Shared Function is64Bit() As Boolean
        Return Marshal.SizeOf(GetType(IntPtr)) = 8
    End Function

    Public Shared Function Initialize() As UInteger
        Return If(is64Bit(), Initialize_64(), Initialize_32())
    End Function

    Public Shared Sub Close()
        If is64Bit() Then
            Close_64()
        Else
            Close_32()
        End If
    End Sub

    Public Shared Function Version() As UInteger
        Return If(is64Bit(), Version_64(), Version_32())
    End Function

    Public Shared Function GetInfo(ByVal DeviceIndex As UInteger, ByRef SerialNumber As UInteger, ByRef Model As Integer) As Boolean
        Return If(is64Bit(), GetInfo_64(DeviceIndex, SerialNumber, Model), GetInfo_32(DeviceIndex, SerialNumber, Model))
    End Function

    Public Shared Function GetKeyString(ByVal DeviceIndex As UInteger, ByVal SwitchIndex As UInteger, ByVal KeyString As StringBuilder, ByVal MaxLength As UInteger) As Boolean
        Return If(is64Bit(), GetKeyString_64(DeviceIndex, SwitchIndex, KeyString, MaxLength), GetKeyString_32(DeviceIndex, SwitchIndex, KeyString, MaxLength))
    End Function

    Public Shared Function GetSwitchSerial(ByVal DeviceIndex As UInteger, ByVal SwitchIndex As UInteger, ByRef SerialNumber As UInteger) As Boolean
        Return If(is64Bit(), GetSwitchSerial_64(DeviceIndex, SwitchIndex, SerialNumber), GetSwitchSerial_32(DeviceIndex, SwitchIndex, SerialNumber))
    End Function

    Public Shared Function IsSwitchDown(ByVal DeviceIndex As UInteger, ByVal SwitchIndex As UInteger) As Boolean
        Return If(is64Bit(), IsSwitchDown_64(DeviceIndex, SwitchIndex), IsSwitchDown_32(DeviceIndex, SwitchIndex))
    End Function

    Public Shared Function SetDisabled(ByVal DeviceIndex As UInteger, ByVal Disabled As UShort) As Boolean
        Return If(is64Bit(), SetDisabled_64(DeviceIndex, Disabled), SetDisabled_32(DeviceIndex, Disabled))
    End Function

    Public Shared Function SetLED(ByVal DeviceIndex As UInteger, ByVal LEDFunc As LEDFunction, ByVal Dimming As UShort) As Boolean
        Return If(is64Bit(), SetLED_64(DeviceIndex, LEDFunc, Dimming), SetLED_32(DeviceIndex, LEDFunc, Dimming))
    End Function

    Public Shared Function SetOutput(ByVal DeviceIndex As UInteger, ByVal OutputIndex As UInteger, ByVal Value As UInteger) As Boolean
        Return If(is64Bit(), SetOutput_64(DeviceIndex, OutputIndex, Value), SetOutput_32(DeviceIndex, OutputIndex, Value))
    End Function

    Public Shared Function SetLayout(ByVal Layout As Integer) As Boolean
        Return If(is64Bit(), SetLayout_64(Layout), SetLayout_32(Layout))
    End Function
End Class
