/* USBHID.H
 *
 * Definitions for the USBHID interface version 1.6.
 * Copyright (c) 2019-2024, CompuPhase, The Netherlands
 */

#ifndef _inc_USBHID_H
#define _inc_USBHID_H

#if defined _WIN32

# if !defined(_INC_WINDOWS) && !defined(__WINDOWS_H)
#   include <windows.h>
# endif

# if !defined(_USBHID_INTERNAL)
#   define USBHIDAPI   WINAPI
# endif

#elif defined __linux || defined __linux__ || defined __APPLE__

# if !defined(_USBHID_INTERNAL)
#   define USBHIDAPI
# endif

  typedef int BOOL;

#endif

#ifdef __cplusplus
extern "C"
{
#endif

enum {                  /* LED functions for usbhid_SetLED() */
  LED_OFF = 0,
  LED_ON,
  LED_BLINK,
};

#if !defined _HID_KEYS_H
enum {
  LAYOUT_BELGIAN = 2,
  LAYOUT_CANADIAN = 4,
  LAYOUT_DANISH = 6,
  LAYOUT_FRENCH = 8,
  LAYOUT_GERMAN = 9,
  LAYOUT_INTERNATIONAL = 13,
  LAYOUT_ITALIAN = 14,
  LAYOUT_LATIN_AMERICAN = 17,
  LAYOUT_NORWEGIAN = 19,
  LAYOUT_PORTUGUESE = 22,
  LAYOUT_SPANISH = 25,
  LAYOUT_SWEDISH = 26,
  LAYOUT_UK = 32,
  LAYOUT_US = 33,
};
#endif

unsigned USBHIDAPI usbhid_Initialize(void);

void USBHIDAPI usbhid_Close(void);

unsigned USBHIDAPI usbhid_Version(void);

BOOL USBHIDAPI usbhid_GetInfo(unsigned DeviceIndex, unsigned *SerialNumber, int *Model);

#if defined __linux || defined __linux__ || defined __APPLE__
BOOL USBHIDAPI usbhid_GetKeyString(unsigned DeviceIndex, unsigned SwitchIndex, char *KeyString, unsigned MaxLength);
#else
BOOL USBHIDAPI usbhid_GetKeyStringA(unsigned DeviceIndex, unsigned SwitchIndex, char *KeyString, unsigned MaxLength);

BOOL USBHIDAPI usbhid_GetKeyStringW(unsigned DeviceIndex, unsigned SwitchIndex, wchar_t *KeyString, unsigned MaxLength);
#endif

BOOL USBHIDAPI usbhid_GetSwitchSerial(unsigned DeviceIndex, unsigned SwitchIndex, unsigned *SerialNumber);

BOOL USBHIDAPI usbhid_IsSwitchDown(unsigned DeviceIndex, unsigned SwitchIndex);

BOOL USBHIDAPI usbhid_PrivilegeError(BOOL Reset);

BOOL USBHIDAPI usbhid_SetDisabled(unsigned DeviceIndex, unsigned short Disabled);

BOOL USBHIDAPI usbhid_SetLED(unsigned DeviceIndex, unsigned short Function, unsigned short Dimming);

BOOL USBHIDAPI usbhid_SetOutput(unsigned DeviceIndex, unsigned OutputIndex, unsigned Value);

BOOL USBHIDAPI usbhid_SetLayout(int layout);

#ifdef __cplusplus
}
#endif

#if defined _WIN32
# if defined UNICODE || defined _UNICODE
#   define usbhid_GetKeyString usbhid_GetKeyStringW
# else
#   define usbhid_GetKeyString usbhid_GetKeyStringA
# endif
#endif

#endif /* _inc_USBHID_H */

